// -*- coding: utf-8 -*-
// Time-stamp: <container.hpp   9 avr 2014 20:13:07> 

#ifndef CONTAINER_H
#define CONTAINER_H

#include <vector>
#include <SFML/Graphics.hpp>
#include "particule.hpp"

class Container {
public:
  // Constructor
  Container(sf::Sprite* sprite, int xmax = 1, int ymax = 1, int n = 10);
  // Destructor
  virtual ~Container();
  // Get X dimension of the container
  double getxdim() const { return xdim; }
  // Get Y dimension of the container
  double getydim() const { return ydim; }
  // Class to encalsulate the iterator of the vector
  // This class hides the implemtation by a vector
  class Iterator {
  public:
    // Constructor from iterator on the vector
    Iterator(std::vector<Particule>::iterator i) : it(i) {} 
    // Delegate equality
    bool operator==(Iterator iterator) { return it == iterator.it; } 
    bool operator!=(Iterator iterator) { return it != iterator.it; } 
    // Delegate incrementation
    Iterator& operator++() { ++it; return *this; }
    // Delegate dereference
    Particule& operator*() { return *it; }
    // Delegate member access
    // Overloading this operator allows to write 
    // it->member instead of (*it).member 
    // where it is an iterator and member is a member of
    // the class iterated that is Particule.
    // The operator returns a pointer to the object and
    // the compiler implicitly applies again the operator -> on it.
    Particule* operator->() { return &*it; }
  private:
    // Iterator on the vector
    std::vector<Particule>::iterator it;
  };
  // Get iterator to beginning
  Iterator begin() { return Iterator(v.begin()); }
  // Get iterator to end
  Iterator end()   { return Iterator(v.end()); }
  // Move all particules in the container
  void move(double speed);
protected:
  // Constructor needed for the derived classes
  // It does not add any particules to the container
  Container(int xmax = 1, int ymax = 1) : xdim(xmax), ydim(ymax) {}
  // Add particule to container
  void add(Particule particule) { v.push_back(particule); }
  // Move one particule in the container
  virtual void move(Particule* particule, double speed) const;

private:
  int xdim;		        // X dimension
  int ydim;		        // Y dimension
  std::vector<Particule> v;	// Particules in the container
};

#endif 
