// -*- coding: utf-8 -*-
// Time-stamp: <experiment.cpp  29 sep 2021 20:49:16> 

#include <iostream>
#include <SFML/Graphics.hpp>
#include "bicontainer.hpp"
#include "experiment.hpp"

Experiment::Experiment()
  : window(sf::VideoMode(xdim, ydim), "Gaz experiment") {

  // Load images in textures and create sprite 
  // Red sprite
  if (!redtexture.loadFromFile("Images/red.gif")) {
    std::cerr << "Error loading Images/red.gif" << std::endl;
    exit(0);
  }
  redsprite.setTexture(redtexture);
  // Blue sprite
  if (!bluetexture.loadFromFile("Images/blue.gif")) {
    std::cerr << "Error loading Images/blue.gif" << std::endl;
    exit(0);
  }
  bluesprite.setTexture(bluetexture);

  // Create container
  container = new Bicontainer(&redsprite, &bluesprite);
  //container = new Container(&bluesprite, 2, 2);
}

// Destructor
Experiment::~Experiment() {
  // Free the container
  delete container;
}

// Run the main loop of the experiment
void Experiment::run() {
  while (window.isOpen()) {
    sf::sleep(sf::milliseconds(10)); // Wait 1/100 second
    processEvents();
    update();
    render();
  }
}

// Process all pending events
void Experiment::processEvents() {
  sf::Event event;
  while (window.pollEvent(event)) {
    if (event.type == sf::Event::Closed)
      window.close();
  }
}

// Update data
void Experiment::update() {
  // Move all particules in container
  container->move(0.01);
}

// Update view
void Experiment::render() {
  // Background color
  window.clear(sf::Color::White);
  // Draw seperation
  sf::RectangleShape rectangle;
  rectangle.setSize(sf::Vector2f(0, 130));
  rectangle.setOutlineColor(sf::Color::Black);
  rectangle.setOutlineThickness(1.5);
  rectangle.setPosition(300, 0);
  window.draw(rectangle);
  rectangle.setPosition(300, 170);
  window.draw(rectangle);
  // Draw a sprite for each particule
  for (auto it = container->begin(); it != container->end(); ++it) {
    // Sprite of the particule
    sf::Sprite* sprite =  it->getSprite();
    // Dimensions of the sprite
    sf::FloatRect rect = sprite->getGlobalBounds();
    // Position of upper-left corner of the sprite
    float xpos = (xdim - rect.width)  * it->getX() / container->getxdim();
    float ypos = (ydim - rect.height) * it->getY() / container->getydim();
    // Set position of the sprite
    sprite->setPosition(xpos, ypos);
    // Draw the spite at the given position
    window.draw(*sprite);
  }
  // Make update visible
  window.display();
}
